# Формирование swagger спецификации на основании метаданных конфигурации 1С.

## Пара ссылок по теме

[Wiki](https://ru.wikipedia.org/wiki/OpenAPI_(%D1%81%D0%BF%D0%B5%D1%86%D0%B8%D1%84%D0%B8%D0%BA%D0%B0%D1%86%D0%B8%D1%8F))

[Описание формата](https://github.com/OAI/OpenAPI-Specification/blob/master/versions/2.0.md)

[Сайт Swagger](https://swagger.io/)

[Swagger Editor (быстро проверить свои jsonчики)](https://editor.swagger.io/)

## Принцип работы библиотеки

На основании выгрузки конфигурации в файлы (в будущем добавлю поддержку edt) производится поиск/парсинг/описание http сервисов и преобразование полученных данных в спецификацию swagger (json на выходе).

Парсинг модуля сервиса предназначен для комментариев вызовов по стандрату [ИТС](https://its.1c.ru/db/v8std#content:453:hdoc)

## Дополнительное описание методов

### Параметры

В блоке параметров может быть описана сложная структура входящих данных. Уровень вложенности параметра определяется количеством знаков * перед именем параметра.

Имя сложного типа должно состоять из одного слова и быть уникальным.

Для описания множественных значений перед названием типа должна быть ключевая фраза *Массив из*.

Для обязательных парметров в тексте описания нужно добавить ключевое слово *обязательный*.

```bsl
// Параметры:
//   userID - Строка - Идентификатор пользователя
//   messageId - Строка - Идентификатор сообщения, обязательный
//   body - Массив из Accept - Информация о подтверждении
//    * orderId - Строка - ID заказа
//    * data - Массив из AdditionalData - Дополнительная информация
//     ** name - Строка - Имя свойства
//     ** value - Массив из Строка - Значения свойства
//    * status - Строка - Статус
//     ~ Подтверждено - Статус "Подтверждено"
//     ~ Отклонено - Статус "Отклонено"
```

### Формат данных

Формат входящих и исходящих данных описывается блоками *Варианты вызова* и *Варианты ответа*:

```bsl
// Варианты вызова:
//   application/json
//   application/xml
//
// Варианты ответа:
//   application/json
//   text/html
```

### Коды ответов

Коды ответа указываются в формате "Код - Описание".

Для каждого кода ответа дополнительно можно определить возвращаемое значение. Для этого код ответа должен быть описан по шаблону "Код - Возвращаемое значение - Описание". 

Поддерживается сложная структура возвращаемых данных.

```bsl
// Коды ответов:
//   200 - Массив из Order - Массив заказов
//    * orderId - Строка - ID заказа
//    * status - Строка - Статус
//    * taskList - Массив из Task - Массив задач
//     ** taskId - Строка - ID задания
//     ** taskNumber - Строка - Номер задания
//   400 - Ошибка выполнения
//   401 - Некорректный токен авторизации
```

## Пример использования библиотеки

```bsl
#Использовать swagger

// класс для парсинга метаданных конфигурации
ПарсерМетаданных = Новый ПарсерМетаданных();

Если ПарсерМетаданных.ПодключитьКонфигурацию(ПутьВыгрузкиКонфигурации, "xml") Тогда

    // без параметров читаем все сервисы, возможен отбор по названию через запятую
    ПарсерМетаданных.ВыполнитьПарсингКонфигурации();

    Для Каждого КЗ Из ПарсерМетаданных.ВнутренниеОписанияСервисов Цикл

        ОписаниеСервиса = КЗ.Значение;
        
        // ОписаниеСервиса["Хост"] = "1c-subs";
        // ОписаниеСервиса["КорневойURL"] = "/perf-metrics/hs" + ОписаниеСервиса["КорневойURL"];

        // класс генерации спецификации, создаем на результате работы ПарсерМетаданных
        ГенераторСпецификации = Новый ГенераторСпецификации(ОписаниеСервиса);

        ГенераторСпецификации.ПрочитатьОписаниеСервиса();

        // ПолучитьСпецификацию() - возвращает текст
        // СохранитьСпецификацию(КаталогВыгрузки, "json") - сохраняет в файл
        Сообщить(ГенераторСпецификации.ПолучитьСпецификацию());

    КонецЦикла;

Иначе
    Сообщить("Не удалось подключить конфигурацию!", СтатусСообщения.Внимание);
КонецЕсли;
```

Пример получения спецификации в скрипте [example_specgenerator.os](https://github.com/botokash/swagger/blob/master/examples/script/example_specgenerator.os)

## Simple-UI

Пример реализация быстрого каталогизатора спецификаций внутри компании. Используем готовый [swagger-ui](https://github.com/swagger-api/swagger-ui), rest сервисы на oscript и IIS.

1. Создаем виртуальный каталог "onec-swagger-ui", закидываем внутрь [dist](https://github.com/swagger-api/swagger-ui/tree/master/dist) из репозитария swagger-ui.
2. Создаем приложение "onec-swagger-admin", внутрь ложим [rest-admin](https://github.com/botokash/swagger/tree/master/simple-ui/rest-admin).
3. Заменяем index.html на [новый](https://github.com/botokash/swagger/blob/master/simple-ui/index.html).
4. В новом index.html указываем в переменной ```admin_url``` свой сервер IIS публикаци rest-admin.
 5. Загрузка спецификаций будет происходить скриптом [upload.os](https://github.com/botokash/swagger/blob/master/simple-ui/upload.os) следующей командой: ```oscript .\upload.os -name ИмяПроекта -path ПутьКВыгрузкеФайлов -type xml -adminurl http://ВашСервер/onec-swagger-admin/```